@@ -15,13 +15,13 @@
 #    under the License.
 
 import collections
-import json
 import logging as real_logging
 import re
 import time
 
 import jsonschema
 from oslo_log import log as logging
+from oslo_serialization import jsonutils as json
 import six
 
 from tempest_lib.common import http
@@ -442,16 +442,9 @@ class RestClient(object):
             self._log_request_full(method, req_url, resp, secs, req_headers,
                                    req_body, resp_body, caller_name, extra)
 
-    def _json_loads(self, resp_body):
-        if isinstance(resp_body, bytes):
-            resp_body = json.loads(resp_body.decode('utf8'))
-        else:
-            resp_body = json.loads(resp_body)
-        return resp_body
-
     def _parse_resp(self, body):
         try:
-            body = self._json_loads(body)
+            body = json.loads(body)
         except ValueError:
             return body
 
@@ -468,7 +461,9 @@ class RestClient(object):
         #  }
         try:
             # Ensure there are not more than one top-level keys
-            if len(body.keys()) > 1:
+            # NOTE(freerunner): Ensure, that JSON is not nullable to
+            # to prevent StopIteration Exception
+            if len(body.keys()) != 1:
                 return body
             # Just return the "wrapped" element
             first_key, first_item = six.next(six.iteritems(body))
@@ -599,6 +594,7 @@ class RestClient(object):
         :raises Forbidden: If a 403 response code is received
         :raises NotFound: If a 404 response code is received
         :raises BadRequest: If a 400 response code is received
+        :raises Gone: If a 410 response code is received
         :raises Conflict: If a 409 response code is received
         :raises OverLimit: If a 413 response code is received and over_limit is
                           not in the response body
@@ -703,6 +699,11 @@ class RestClient(object):
                 resp_body = self._parse_resp(resp_body)
             raise exceptions.BadRequest(resp_body, resp=resp)
 
+        if resp.status == 410:
+            if parse_resp:
+                resp_body = self._parse_resp(resp_body)
+            raise exceptions.Gone(resp_body, resp=resp)
+
         if resp.status == 409:
             if parse_resp:
                 resp_body = self._parse_resp(resp_body)
