@@ -244,6 +244,7 @@ class TestRestClientParseRespJSON(BaseRestClientTestClass):
         keys[0]: values[0],
         keys[1]: values[1],
     }}
+    null_dict = {}
 
     def setUp(self):
         self.fake_http = fake_http.fake_httplib2()
@@ -273,6 +274,10 @@ class TestRestClientParseRespJSON(BaseRestClientTestClass):
         body = self.rest_client._parse_resp(json.dumps(data))
         self.assertEqual(data, body)
 
+    def test_parse_nullable_dict(self):
+        body = self.rest_client._parse_resp(json.dumps(self.null_dict))
+        self.assertEqual(self.null_dict, body)
+
 
 class TestRestClientErrorCheckerJSON(base.TestCase):
     c_type = "application/json"
@@ -330,6 +335,9 @@ class TestRestClientErrorCheckerJSON(base.TestCase):
     def test_response_409(self):
         self._test_error_checker(exceptions.Conflict, self.set_data("409"))
 
+    def test_response_410(self):
+        self._test_error_checker(exceptions.Gone, self.set_data("410"))
+
     def test_response_413(self):
         self._test_error_checker(exceptions.OverLimit, self.set_data("413"))
 
@@ -405,6 +413,25 @@ class TestRestClientErrorCheckerJSON(base.TestCase):
         expected = r_body
         self.assertEqual(expected, e.resp_body)
 
+    def test_response_410_with_dict(self):
+        r_body = '{"resp_body": {"err": "fake_resp_body"}}'
+        e = self._test_error_checker(exceptions.Gone,
+                                     self.set_data("410", r_body=r_body))
+
+        if self.c_type == 'application/json':
+            expected = {"err": "fake_resp_body"}
+        else:
+            expected = r_body
+        self.assertEqual(expected, e.resp_body)
+
+    def test_response_410_with_invalid_dict(self):
+        r_body = '{"foo": "bar"]'
+        e = self._test_error_checker(exceptions.Gone,
+                                     self.set_data("410", r_body=r_body))
+
+        expected = r_body
+        self.assertEqual(expected, e.resp_body)
+
     def test_response_409_with_dict(self):
         r_body = '{"resp_body": {"err": "fake_resp_body"}}'
         e = self._test_error_checker(exceptions.Conflict,
